% FILTERING Learn about Fourier filtering with MATLAB
% This function provides some supplementary material to the IOW notes on
% spectral analysis. 
%
% Info: lars.umlauf@io-warnemuende.de


%% define parameters

T        = 1;            % period
N        = 100;           % number of sampling intervals
dt       = T/N;           % sampling interval
fs       = 1/dt;          % sampling frequency
df       = 1/T;           % frequency resolution

% filter specification
FilterType =  'bart';
Delta      = 20*dt;

% other stuff
FontSize = 12;            % for plotting


%% create some sinusoidal data

% compose time vecors
t        = [0:dt:T-dt]';

% generate input timeseries
n       = 5;                             % mode number
w1      = 2*pi*n/T;                      % frequency in rad/s
u       = 1 + cos(w1*t);


%% compute number of represented frequencies

% index of hightest representable frequency in two-sided spectra
% This depends on the way fft() deals with even and odd record numbers
Nup = N/2;
if round(Nup) == Nup 
   % even   
   Nup   = N/2 + 1;
   f     = [0:df:(Nup-1)*df]';  
   f     = [-flipud(f) ; f(2:end-1)];
   even  = 1;
else
    % odd
   Nup   = (N+1)/2;
   f     = [0:df:(Nup-1)*df]';  
   f     = [-flipud(f) ; f(2:end)];
   even  = 0;
end

% compute frequency vector (rad/s)
omega    = 2*pi*f;


%% create filtering window

switch lower(FilterType)
    case 'box'
       gHat       = sin(omega*Delta/2) ./ (omega*Delta/2);
    case 'bart'
       gHat       = ( sin(omega*Delta/4) ./ (omega*Delta/4) ).^2;
    otherwise
        error('unkown FilterType')
end

gHat(Nup)  = 1; 


%% filtering

% everything rescaled by 1/N and shifted to the notation used in the notes

% compute FFT of the signal
uHat    = 1/N*fft(u);
uHat   = fftshift(uHat);

% spectrum is filtered by multiplying with FFT of window function
ufHat  = uHat .* gHat;

% compute inverse FFT to get the filtered signal in physical space
g    = ifft(N*ifftshift(gHat));
uf   = ifft(N*ifftshift(ufHat));


%% plotting

nFig = 0;

% time series
nFig = nFig + 1;
figure(nFig);
clf;

subplot(2,1,1)
plot(t,u,'o-k',t,uf,'o-r')
grid on;

legend('raw','filtered')
set(gca,'FontSize',FontSize,'XTickLabel',{})
ylabel('u(t)')
title('Signals');

subplot(2,1,2)
plot(t,g,'o-k')

grid on;
set(gca,'FontSize',FontSize)
xlabel('t (s)')
ylabel('g(t)')
title('Filter Window');


% Fourier coefficients
nFig = nFig + 1;
figure(nFig);
clf;

subplot(3,1,1)
plot(omega,real(uHat),'o-k')

grid on;
set(gca,'FontSize',FontSize,'XTickLabel',{})
ylabel('$\Re(\hat{u})$','Interpreter','latex')
title('Real Part');

subplot(3,1,2)
plot(omega,imag(uHat),'o-k');

grid on;
set(gca,'FontSize',FontSize,'XTickLabel',{})
ylabel('$\Im(\hat{u})$','Interpreter','latex')
title('Imaginary Part')

subplot(3,1,3)
plot(omega,gHat,'o-k');

grid on;
set(gca,'FontSize',FontSize)
ylabel('$\hat{g}$','Interpreter','latex')
xlabel('\omega (rad/s)')
title('Filter Window')

